import pandas as pd
import numpy as np
import datetime as dt
import xgboost as xgb
import matplotlib.pyplot as plt
import seaborn as sns
import statsmodels.api as sm


sns.set()


class Dos():
    """ 
    Class Dos is  a class that could daily predect and plot three sources 
    of three sources of enrergy: Electricity Consumption, Gaz and Nuclear.

    
    :param data: This is the data training, it is the output of :ref:`Data Collection <DataCol>`.
    :type data: Dataframe

    :param energy: This model predict three sources of energy: Electricity Consumption, Gaz and Nuclear.
            0 : Electricity Consumption
            1 : Gaz
            2: Nuclear
    :type energy: int

    :param Year: The Year for which day we want to predict.
    :type Year: int

    :param month: The month for which day we want to predict.
    :type month: int
    
    :param day: The day for which day we want to predict.
    :type day: int
    """
    def __init__(self, data, energy, year, month, day):
        self.data = data
        self.energie = energy
        self.year = year
        self.month = month
        self.day = day
        
    def createFeatures(self):
        """
        Create time series features based on time series index.
        """
        target = self.data.columns[self.energie]
        target_map = self.data[target].to_dict()
        self.data = self.data.copy()
        ind = self.data.index
        self.data['minute'] = ind.minute
        self.data['dayofweek'] = ind.dayofweek
        self.data['month'] = ind.month
        self.data['year'] = ind.year
        self.data['dayofyear'] = ind.dayofyear
        self.data['dayofmonth'] = ind.day
        self.data['lag1'] = (ind - pd.Timedelta('364 days')).map(target_map)
        self.data['lag2'] = (ind - pd.Timedelta('30 days')).map(target_map)
        self.data['lag3'] = (ind - pd.Timedelta('7 days')).map(target_map)
        return self.data

    def fitModel(self):
        """ Automatic process that makes sure our machine learning models have high 
        level of accuracy. The Model name is **Boosted Trees** for more 
        information please see this `referance 
        <https://xgboost.readthedocs.io/en/stable/tutorials/model.html>`_"""
        target = self.data.columns[self.energie]
        FEATURES = ['dayofyear', 'minute', 'dayofweek', 'month', 'year',
                    'lag1', 'lag2', 'lag3']
        TARGET = target
        X_all = self.data[FEATURES]
        y_all = self.data[TARGET]
        reg = xgb.XGBRegressor(base_score=0.5,
                               booster='gbtree',
                               n_estimators=500,
                               objective='reg:linear',
                               max_depth=3,
                               learning_rate=0.01)
        reg.fit(X_all, y_all,
                eval_set=[(X_all, y_all)],
                verbose=100)

        return reg

    def DayPred(self, reg):
        """
        Predict confidence scores for samples.
        The confidence score for a sample is proportional to the signed
        distance of that sample to the hyperplane.

        :param reg: this is the output of **fitModel** .
        :type Year: xgboost.sklearn.XGBRegressor



        """
        liste = []
        for i in range(96):
            liste.append(str(dt.datetime(self.year, self.month,
                         self.day, 0, 0) + dt.timedelta(minutes=(i)*15)))
        DayDate = pd.to_datetime(np.array(liste))

        dayFeaturs = pd.DataFrame()
        dayFeaturs['dayofyear'] = DayDate.dayofyear
        dayFeaturs['minute'] = DayDate.minute
        dayFeaturs['dayofweek'] = DayDate.dayofweek
        dayFeaturs['month'] = DayDate.month
        dayFeaturs['year'] = DayDate.year

        target = self.data.columns[self.energie]
        target_map = self.data[target].to_dict()
        dayFeaturs['lag1'] = (
            DayDate - pd.Timedelta('364 days')).map(target_map)
        dayFeaturs['lag2'] = (
            DayDate - pd.Timedelta('30 days')).map(target_map)
        dayFeaturs['lag3'] = (DayDate - pd.Timedelta('7 days')).map(target_map)

        pred = reg.predict(dayFeaturs)
        Date = DayDate.strftime('%Y-%m-%d')
        Hour = DayDate.strftime('%H:%M')
        day_pred = pd.DataFrame()
        day_pred["Date"] = Date
        day_pred["Heure"] = Hour
        day_pred[target] = pred

        return day_pred, DayDate

    def plot(self, day_pred, DayDate):
        """ This method plot daily energy predection indexing by Time """
        target = self.data.columns[self.energie]

        day_pred = day_pred[target].values
        day_pred = pd.DataFrame(day_pred, index=DayDate,
                                columns=['{}'.format(target)])
        f, ax = plt.subplots(figsize=(12, 6), dpi=200)
        plt.suptitle('{}-{}-{}, forecasting {}'.format(self.year,
                     self.month, self.day, target), fontsize=24)
        day_pred.plot(ax=ax, rot=90, ylabel='MW', legend="Predicted day ")
        plt.show()



#%%
import os
import pandas as pd
import pooch
from abdu.ot import url_dc, path_target


class Load_db:
    def __init__(self, url=url_dc, target_name=path_target):
        path, fname = os.path.split(path_target)
        pooch.retrieve(url, path=path, fname=fname, known_hash=None)

    @staticmethod
    def save_as_df():
        df_bikes = pd.read_csv(
            path_target,
            na_values="",
            low_memory=False,
            converters={"data": str, "heure": str},
        )
        return df_bikes

# %%
df = Load_db().save_as_df()


import os

url_dc = "https://raw.githubusercontent.com/otmaneelallaki/HAX712X-DOS/main/Project/Prediction/Dataset/DataTrining.csv"
path_target = os.path.join(
    os.path.dirname(os.path.realpath(__file__)), "..", "data", "bicycle_db.csv"
)



import pandas as pd
import datetime
import numpy as np
import pooch  # download data / avoid re-downloading
import os
import sys


class Data():
    """ 
    This class is for downloading  the data 
    from `2012 into 2022 <https://bit.ly/3UyWiN4>`_  
    and fil out all the missing values 
    """

    def __init__(self, data=pd.DataFrame()) -> None:
        """ 
        initalisation by empty data dataFrame 
        """
        self.data = data

    def impo(self):
        """ 
        This Method is for importing data from the package: this data is alrady 
        clean and filtered by using the **dataDownload** method 

        :return: to a data trining that will be imported after to train our
         Model forcast.

        :rtype: DataFrame
        """
        self.data = pd.read_csv(os.path.abspath(
            '../Project/Prediction/Dataset/DataTrining.csv'))
        return self.data

    def dataDownload(self):
        """
        This method is for download the data from 2019 to 2022 year by year.
         """
        # loading raw data 2019
        url2019 = "https://bit.ly/3hVlwrl"
        path_target = "./eco2mix-national-cons-def2019.csv"
        path, fname = os.path.split(path_target)
        # if needed `pip install pooch`
        pooch.retrieve(url2019, path=path, fname=fname, known_hash=None)
        # variables we needs
        # importin g our data
        usecols2019 = ['Date', 'Heure',  
                       "Consommation (MW)", "Nucléaire (MW)", "Gaz (MW)"]
        data2019 = pd.read_csv("./eco2mix-national-cons-def2019.csv",
                               usecols=usecols2019, sep=";")

        # loading raw data 2020
        url2020 = "https://bit.ly/3tLbMCE"
        path_target = "./eco2mix-national-cons-def2020.csv"
        path, fname = os.path.split(path_target)
        # if needed `pip install pooch`
        pooch.retrieve(url2020, path=path, fname=fname, known_hash=None)
        # variables we needs
        usecols2020 = ['Date', 'Heure',
                       "Consommation (MW)", "Nucléaire (MW)", "Gaz (MW)"]
        data2020 = pd.read_csv("./eco2mix-national-cons-def2020.csv",
                               usecols=usecols2020, sep=";")  

        # loading raw data 2021
        url2021 = "https://bit.ly/3tId29N"
        path_target = "./eco2mix-national-cons-def2021.csv"
        path, fname = os.path.split(path_target)
        # if needed `pip install pooch`
        pooch.retrieve(url2021, path=path, fname=fname, known_hash=None)
        # variables we needs
        usecols2021 = ['Date', 'Heure',
                       "Consommation (MW)", "Nucléaire (MW)", "Gaz (MW)"]
        data2021 = pd.read_csv("./eco2mix-national-cons-def2021.csv",
                               usecols=usecols2021, sep=";")  

        # loading  half raw data 2022
        url2022half = "https://bit.ly/3gisk1Y"
        path_target = "./eco2mix-national-cons-def_half2022.csv"
        path, fname = os.path.split(path_target)
        pooch.retrieve(url2022half, path=path, fname=fname,
                       known_hash=None)  # if needed `pip install pooch`
        # variables we needs
        usecolshalf2022 = ['Date', 'Heure',
                           "Consommation (MW)", "Nucléaire (MW)", "Gaz (MW)"]
        dataHalf2022 = pd.read_csv("./eco2mix-national-cons-def_half2022.csv",
                                   usecols=usecolshalf2022, sep=";")  

        # loading raw data 2022
        url2022 = "https://bit.ly/3VgREnT"
        path_target = "./eco2mix-national-cons-def2022.csv"
        path, fname = os.path.split(path_target)
        # if needed `pip install pooch`
        pooch.retrieve(url2022, path=path, fname=fname, known_hash=None)
        # variables we needs
        usecols2022 = ['Date', 'Heure',
                       "Consommation (MW)", "Nucléaire (MW)", "Gaz (MW)"]
        data2022 = pd.read_csv("./eco2mix-national-cons-def2022.csv",
                               usecols=usecols2022, sep=";")  
        data2022.dropna(inplace=True)
        print(data2022.isna().sum())

        # _____________________________________________________________
        time_improved = pd.to_datetime(data2019['Date'] +
                                       ' ' + data2019['Heure'],
                                       format='%Y-%m-%d %H:%M')
        data2019['Time'] = time_improved  # add the Time to the data
        data2019.set_index('Time', inplace=True)  # set time as index
        # remove useles columns
        del data2019['Heure']
        del data2019['Date']
        data2019 = data2019.sort_index(ascending=True)

        time_improved = pd.to_datetime(data2020['Date'] +
                                       ' ' + data2020['Heure'],
                                       format='%Y-%m-%d %H:%M')
        data2020['Time'] = time_improved  # add the Time to the data
        data2020.set_index('Time', inplace=True)  # set time as index
        # remove useles columns
        del data2020['Heure']
        del data2020['Date']
        data2020 = data2020.sort_index(ascending=True)

        time_improved = pd.to_datetime(data2021['Date'] +
                                       ' ' + data2021['Heure'],
                                       format='%Y-%m-%d %H:%M')
        data2021['Time'] = time_improved  # add the Time to the data
        data2021.set_index('Time', inplace=True)  # set time as index
        # remove useles columns
        del data2021['Heure']
        del data2021['Date']
        data2021 = data2021.sort_index(ascending=True)

        time_improved = pd.to_datetime(dataHalf2022['Date'] +
                                       ' ' + dataHalf2022['Heure'],
                                       format='%Y-%m-%d %H:%M')
        dataHalf2022['Time'] = time_improved  # add the Time to the data
        dataHalf2022.set_index('Time', inplace=True)  # set time as index
        # remove useles columns
        del dataHalf2022['Heure']
        del dataHalf2022['Date']
        dataHalf2022 = dataHalf2022.sort_index(ascending=True)

        time_improved = pd.to_datetime(data2022['Date'] +
                                       ' ' + data2022['Heure'],
                                       format='%Y-%m-%d %H:%M')
        data2022['Time'] = time_improved  # add the Time to the data
        data2022.set_index('Time', inplace=True)  # set time as index
        # remove useles columns
        del data2022['Heure']
        del data2022['Date']
        data2022 = data2022.sort_index(ascending=True)

        # --------------------------------------------------
        frames = [data2019, data2020, data2021, dataHalf2022, data2022]
        self.data = pd.concat(frames)
        return self.data

    def Filnan(self):
        """ 
        In this Method we have fill the nan by the mean of two successive observation:

        .. warning::
            If you use **dataDownloadThis()** method then it's very important
            to call this method directly, otherwise you will get the data full of *nan*
            values.
            """ 
        for nan in range(np.shape(self.data)[0]-1):    # fil nan  2010
            if self.data[["Consommation (MW)"]].isna().iloc[:, 0][nan]:
                # replace nan by the mean between 2 samples
                self.data.loc[:, "Gaz (MW)"][nan] = (
                    self.data.loc[:, "Gaz (MW)"][nan-1] + 
                    self.data.loc[:, "Gaz (MW)"][nan+1])/2
                self.data.loc[:, "Nucléaire (MW)"][nan] = (self.data.loc[:, "Nucléaire (MW)"][nan-1] +
                                                           self.data.loc[:, "Nucléaire (MW)"][nan+1])/2  
                self.data.loc[:, "Consommation (MW)"][nan] = (self.data.loc[:, "Consommation (MW)"][nan-1] +
                                                              self.data.loc[:, "Consommation (MW)"][nan+1])/2  
        self.data.loc[:, "Consommation (MW)"][nan +
                                              1] = self.data.loc[:, "Consommation (MW)"][nan]
        return self.data

